package com.jprofiler.api.agent;

/**
 * Heap dump options for calling {@link Controller#triggerHeapDump(HeapDumpOptions)}
 * The default values after constructing an instance of this class are equivalent to calling
 * <tt>heapDumpOptions.fullGc(true).primitiveData(true).calculateRetainedSizes(true).selectRecorded(true)</tt>.
 */
public class HeapDumpOptions {

    /**
     * The default value used by {@link Controller#triggerHeapDump()}. This uses the default options described above.
     */
    public static final HeapDumpOptions DEFAULT = new HeapDumpOptions().freeze();

    /**
     * Same as {@link #DEFAULT}, only with <tt>selectRecorded(true)</tt>.
     */
    public static final HeapDumpOptions SELECT_RECORDED = new HeapDumpOptions().selectRecorded(true).freeze();

    /**
     * Same as {@link #DEFAULT}, only with <tt>fullGc(false)</tt>.
     */
    public static final HeapDumpOptions NO_FULL_GC = new HeapDumpOptions().fullGc(false).freeze();

    private boolean fullGc = true;
    private boolean primitiveData = true;
    private boolean calculateRetainedSizes = true;
    private boolean selectRecorded = false;

    private boolean frozen = false;

    /**
     * Create a new instance with the default options described above.
     */
    public HeapDumpOptions() {
    }

    /**
     * Determines whether a full garbage collection should be performed for the heap dump.
     * The default is <tt>true</tt>.
     * @param fullGc the new value
     * @return this instance
     */
    public HeapDumpOptions fullGc(boolean fullGc) {
        checkFrozen();
        this.fullGc = fullGc;
        return this;
    }

    /**
     * Determines whether primitive data should be recorded for the heap dump.
     * The default is <tt>true</tt>.
     * @param primitiveData the new value
     * @return this instance
     */
    public HeapDumpOptions primitiveData(boolean primitiveData) {
        checkFrozen();
        this.primitiveData = primitiveData;
        return this;
    }

    /**
     * Determines whether retained sizes should be calculated for the heap dump.
     * The default is <tt>true</tt>.
     * @param calculateRetainedSizes the new value
     * @return this instance
     */
    public HeapDumpOptions calculateRetainedSizes(boolean calculateRetainedSizes) {
        checkFrozen();
        this.calculateRetainedSizes = calculateRetainedSizes;
        return this;
    }

    /**
     * Determines whether recorded objects should be selected when the heap dump is opened in the JProfiler GUI.
     * The default value is <tt>false</tt>.
     * @param selectRecorded the new value
     * @return this instance
     */
    public HeapDumpOptions selectRecorded(boolean selectRecorded) {
        checkFrozen();
        this.selectRecorded = selectRecorded;
        return this;
    }

    /**
     * Returns if a full garbage collection should be performed for the heap dump.
     * @return the value
     */
    public boolean isFullGc() {
        return fullGc;
    }

    /**
     * Returns if primitive data should be recorded for the heap dump.
     * @return the value
     */
    public boolean isPrimitiveData() {
        return primitiveData;
    }

    /**
     * Returns if retained sizes should be calculated for the heap dump.
     * @return the value
     */
    public boolean isCalculateRetainedSizes() {
        return calculateRetainedSizes;
    }

    /**
     * Returns if recorded objects should be selected when the heap dump is opened in the JProfiler GUI.
     * @return the value
     */
    public boolean isSelectRecorded() {
        return selectRecorded;
    }

    /**
     * Make these heap dump options unmodifiable. Further modification attempts will result in a
     * <tt>UnsupportedOperationException</tt>.
     * @return this instance
     */
    public HeapDumpOptions freeze() {
        frozen = true;
        return this;
    }

    private void checkFrozen() {
        if (frozen) {
            throw new UnsupportedOperationException("parameters are frozen");
        }
    }
}
