package com.jprofiler.api.agent;

/**
 * Request tracking options for calling {@link Controller#startCPURecording(boolean, TrackingOptions)}} and
 * {@link Controller#startAllocRecording(boolean, TrackingOptions)}. Please see the help topic on request
 * tracking for more information.
 * After constructing an instance of this class, all request tracking options are set to <tt>false</tt>.
 */
public class TrackingOptions {

    /**
     * The default value used by {@link Controller#startCPURecording(boolean)} and {@link Controller#startAllocRecording(boolean, TrackingOptions)}
     */
    public static final TrackingOptions NONE = new TrackingOptions().freeze();

    /**
     * Only track thread start, same as <tt>new TrackingOptions().threadStart(true)</tt>.
     */
    public static final TrackingOptions THREAD_START = new TrackingOptions().threadStart(true).freeze();

    /**
     * Only track executors, same as <tt>new TrackingOptions().executors(true)</tt>.
     */
    public static final TrackingOptions EXECUTORS = new TrackingOptions().executors(true).freeze();

    /**
     * Only track AWT, same as <tt>new TrackingOptions().awt(true)</tt>.
     */
    public static final TrackingOptions AWT = new TrackingOptions().awt(true).freeze();

    /**
     * Only track SWT, same as <tt>new TrackingOptions().swt(true)</tt>.
     */
    public static final TrackingOptions SWT = new TrackingOptions().swt(true).freeze();

    /**
     * Only track RMI, same as <tt>new TrackingOptions().rmi(true)</tt>.
     */
    public static final TrackingOptions RMI = new TrackingOptions().rmi(true).freeze();

    /**
     * Only track remote EJBs, same as <tt>new TrackingOptions().remoteEjb(true)</tt>.
     */
    public static final TrackingOptions REMOTE_EJB = new TrackingOptions().remoteEjb(true).freeze();

    /**
     * Only track web services, same as <tt>new TrackingOptions().webServices(true)</tt>.
     */
    public static final TrackingOptions WEB_SERVICES = new TrackingOptions().webServices(true).freeze();

    private boolean threadStart = false;
    private boolean executors = false;
    private boolean awt = false;
    private boolean swt = false;
    private boolean rmi  = false;
    private boolean remoteEjb = false;
    private boolean webServices = false;

    private boolean frozen = false;

    /**
     * Create a new instance with no tracking types enabled.
     */
    public TrackingOptions() {
    }

    /**
     * Determines if thread start should be tracked.
     * @param threadStart the new value
     * @return this instance
     */
    public TrackingOptions threadStart(boolean threadStart) {
        checkFrozen();
        this.threadStart = threadStart;
        return this;
    }

    /**
     * Determines if executors should be tracked.
     * @param executors the new value
     * @return this instance
     */
    public TrackingOptions executors(boolean executors) {
        checkFrozen();
        this.executors = executors;
        return this;
    }

    /**
     * Determines if AWT should be tracked.
     * @param awt the new value
     * @return this instance
     */
    public TrackingOptions awt(boolean awt) {
        checkFrozen();
        this.awt = awt;
        return this;
    }

    /**
     * Determines if SWT should be tracked.
     * @param swt the new value
     * @return this instance
     */
    public TrackingOptions swt(boolean swt) {
        checkFrozen();
        this.swt = swt;
        return this;
    }

    /**
     * Determines if RMI should be tracked.
     * @param rmi the new value
     * @return this instance
     */
    public TrackingOptions rmi(boolean rmi) {
        checkFrozen();
        this.rmi = rmi;
        return this;
    }

    /**
     * Determines if web services should be tracked.
     * @param webServices the new value
     * @return this instance
     */
    public TrackingOptions webServices(boolean webServices) {
        checkFrozen();
        this.webServices = webServices;
        return this;
    }

    /**
     * Determines if remote EJBs should be tracked.
     * @param remoteEjb the new value
     * @return this instance
     */
    public TrackingOptions remoteEjb(boolean remoteEjb) {
        checkFrozen();
        this.remoteEjb = remoteEjb;
        return this;
    }

    /**
     * Returns if thread start is tracked.
     * @return the value
     */
    public boolean isThreadStart() {
        return threadStart;
    }

    /**
     * Returns if executors are tracked.
     * @return the value
     */
    public boolean isExecutors() {
        return executors;
    }

    /**
     * Returns if AWT is tracked.
     * @return the value
     */
    public boolean isAwt() {
        return awt;
    }

    /**
     * Returns if SWT is tracked.
     * @return the value
     */
    public boolean isSwt() {
        return swt;
    }

    /**
     * Returns if remote EJBs are tracked.
     * @return the value
     */
    public boolean isRemoteEjb() {
        return remoteEjb;
    }

    /**
     * Returns if web services are tracked.
     * @return the value
     */
    public boolean isWebServices() {
        return webServices;
    }

    /**
     * Returns if RMI is tracked.
     * @return the value
     */
    public boolean isRmi() {
        return rmi;
    }

    /**
     * Make these tracking options unmodifiable. Further modification attempts will result in a
     * <tt>UnsupportedOperationException</tt>.
     * @return this instance
     */
    public TrackingOptions freeze() {
        frozen = true;
        return this;
    }

    private void checkFrozen() {
        if (frozen) {
            throw new UnsupportedOperationException("parameters are frozen");
        }
    }
}
